@@ -33,27 +33,33 @@ from renku.core.dataset.datasets_provenance import DatasetsProvenance
 from renku.core.dataset.pointer_file import create_external_file, is_external_file_updated, update_external_file
 from renku.core.dataset.providers.factory import ProviderFactory
 from renku.core.dataset.providers.models import ProviderDataset
+from renku.core.dataset.providers.s3 import S3Credentials
 from renku.core.dataset.request_model import ImageRequestModel
 from renku.core.dataset.tag import get_dataset_by_tag, prompt_access_token, prompt_tag_selection
 from renku.core.interface.client_dispatcher import IClientDispatcher
 from renku.core.interface.dataset_gateway import IDatasetGateway
+from renku.core.interface.storage import IStorageFactory
 from renku.core.util import communication
 from renku.core.util.datetime8601 import local_now
 from renku.core.util.dispatcher import get_client, get_database
 from renku.core.util.git import clone_repository, get_cache_directory_for_repository, get_git_user
-from renku.core.util.metadata import is_external_file
-from renku.core.util.os import delete_file
+from renku.core.util.metadata import is_external_file, prompt_for_credentials, read_credentials, store_credentials
+from renku.core.util.os import (
+    create_symlink,
+    delete_dataset_file,
+    delete_path,
+    get_absolute_path,
+    get_safe_relative_path,
+    hash_file,
+    is_path_empty,
+    is_subpath,
+    unmount_path,
+)
 from renku.core.util.tabulate import tabulate
 from renku.core.util.urls import get_slug
 from renku.core.util.util import NO_VALUE, NoValueType
-from renku.domain_model.dataset import (
-    Dataset,
-    DatasetDetailsJson,
-    DatasetFile,
-    RemoteEntity,
-    get_dataset_data_dir,
-    is_dataset_name_valid,
-)
+from renku.domain_model.dataset import Dataset, DatasetDetailsJson, DatasetFile, RemoteEntity, is_dataset_name_valid
+from renku.domain_model.enums import ConfigFilter
 from renku.domain_model.provenance.agent import Person
 from renku.domain_model.provenance.annotation import Annotation
 from renku.infrastructure.immutable import DynamicProxy
@@ -86,6 +92,7 @@ def list_datasets():
         dataset = DynamicProxy(dataset)
         dataset.tags = tags
         dataset.tags_csv = ",".join(tag.name for tag in tags)
+        dataset.datadir_path = str(dataset.get_datadir())
         datasets.append(dataset)
 
     return list(datasets)
@@ -100,6 +107,8 @@ def create_dataset(
     images: Optional[List[ImageRequestModel]] = None,
     update_provenance: bool = True,
     custom_metadata: Optional[Dict[str, Any]] = None,
+    storage: Optional[str] = None,
+    datadir: Optional[Path] = None,
 ):
     """Create a dataset.
 
@@ -113,6 +122,8 @@ def create_dataset(
         update_provenance(bool, optional): Whether to add this dataset to dataset provenance
             (Default value = True).
         custom_metadata(Optional[Dict[str, Any]], optional): Custom JSON-LD metadata (Default value = None).
+        storage(Optional[str], optional): Backend storage's URI (Default value = None).
+        datadir(Optional[Path]): Dataset's data directory (Default value = None).
 
     Returns:
         Dataset: The created dataset.
@@ -128,22 +139,27 @@ def create_dataset(
 
     if not is_dataset_name_valid(name):
         valid_name = get_slug(name, lowercase=False)
-        raise errors.ParameterError(f'Dataset name "{name}" is not valid (Hint: "{valid_name}" is valid).')
+        raise errors.ParameterError(f"Dataset name '{name}' is not valid (Hint: '{valid_name}' is valid).")
 
     datasets_provenance = DatasetsProvenance()
 
     if datasets_provenance.get_by_name(name=name):
-        raise errors.DatasetExistsError(f"Dataset exists: '{name}'")
+        raise errors.DatasetExistsError(name)
 
     if not title:
         title = name
 
     keywords = keywords or []
 
-    annotations = None
+    annotations = (
+        [Annotation(id=Annotation.generate_id(), source="renku", body=custom_metadata)] if custom_metadata else None
+    )
 
-    if custom_metadata:
-        annotations = [Annotation(id=Annotation.generate_id(), source="renku", body=custom_metadata)]
+    if datadir:
+        try:
+            datadir = get_safe_relative_path(datadir, client.path)
+        except ValueError as e:
+            raise errors.ParameterError("Datadir must be inside repository.") from e
 
     dataset = Dataset(
         identifier=None,
@@ -154,11 +170,17 @@ def create_dataset(
         keywords=keywords,
         project_id=client.project.id,
         annotations=annotations,
+        storage=storage,
+        datadir=datadir,
     )
 
     if images:
         set_dataset_images(client, dataset, images)
 
+    if storage:
+        provider = ProviderFactory.get_create_provider(uri=storage)
+        provider.on_create(dataset=dataset)
+
     if update_provenance:
         datasets_provenance.add_or_update(dataset)
 
@@ -325,8 +347,28 @@ def file_unlink(name, include, exclude, client_dispatcher: IClientDispatcher, ye
         )
         communication.confirm(prompt_text, abort=True, warning=True)
 
+    dataset_datadir = dataset.get_datadir()
     for file in records:
         dataset.unlink_file(file.entity.path)
+        path_file = Path(file.entity.path)
+        # INFO: Remove actual dataset file only if it is located within dataset directory
+        if str(path_file.absolute()).startswith(str(dataset_datadir.absolute())):
+            if not path_file.exists():
+                communication.warn(
+                    f"Dataset file {path_file} could not be found, skipping the removal from {dataset_datadir}."
+                )
+                continue
+            try:
+                if path_file.is_dir():
+                    shutil.rmtree(str(path_file.absolute()), ignore_errors=False, onerror=None)
+                else:
+                    path_file.unlink()
+            except Exception as err:
+                communication.warn(
+                    f"Dataset file {path_file} could not be removed from {dataset_datadir} because of {err}."
+                )
+            else:
+                client.repository.add(path_file)
 
     datasets_provenance.add_or_update(dataset, creator=get_git_user(client.repository))
 
@@ -362,7 +404,7 @@ def export_dataset(name, provider_name, tag, client_dispatcher: IClientDispatche
     # TODO: all these callbacks are ugly, improve in #737
     config_key_secret = "access_token"
 
-    dataset = datasets_provenance.get_by_name(name, strict=True, immutable=True)
+    dataset: Optional[Dataset] = datasets_provenance.get_by_name(name, strict=True, immutable=True)
 
     provider = ProviderFactory.from_name(provider_name)
 
@@ -383,14 +425,15 @@ def export_dataset(name, provider_name, tag, client_dispatcher: IClientDispatche
         if not dataset:
             raise errors.DatasetNotFound(message=f"Cannot find dataset with id: '{selected_tag.dataset_id.value}'")
 
-    data_dir = get_dataset_data_dir(client, dataset.name)  # type: ignore
     dataset = cast(Dataset, DynamicProxy(dataset))
+
+    data_dir = dataset.get_datadir()
     dataset.data_dir = data_dir
 
     exporter = provider.get_exporter(dataset=dataset, tag=selected_tag, **kwargs)
 
     if exporter.requires_access_token():
-        access_token = client.get_value(provider_name, config_key_secret)
+        access_token = read_credentials(section=provider_name, key=config_key_secret)
 
         if access_token is None:
             access_token = prompt_access_token(exporter)
@@ -398,7 +441,7 @@ def export_dataset(name, provider_name, tag, client_dispatcher: IClientDispatche
             if access_token is None or len(access_token) == 0:
                 raise errors.InvalidAccessToken()
 
-            client.set_value(provider_name, config_key_secret, access_token, global_only=True)
+            store_credentials(section=provider_name, key=config_key_secret, value=access_token)
 
         exporter.set_access_token(access_token)
 
@@ -412,28 +455,32 @@ def export_dataset(name, provider_name, tag, client_dispatcher: IClientDispatche
 
 
 def import_dataset(
-    uri,
-    name="",
-    extract=False,
-    yes=False,
-    previous_dataset=None,
-    delete=False,
-    gitlab_token=None,
+    uri: str,
+    name: str = "",
+    extract: bool = False,
+    yes: bool = False,
+    datadir: Optional[Path] = None,
+    previous_dataset: Optional[Dataset] = None,
+    delete: bool = False,
+    gitlab_token: Optional[str] = None,
     **kwargs,
 ):
     """Import data from a 3rd party provider or another renku project.
 
     Args:
-        uri: DOI or URL of dataset to import.
-        name: Name to give imported dataset (Default value = "").
-        extract: Whether to extract compressed dataset data (Default value = False).
-        yes: Whether to skip user confirmation (Default value = False).
-        previous_dataset: Previously imported dataset version (Default value = None).
-        delete: Whether to delete files that don't exist anymore (Default value = False).
-        gitlab_token: Gitlab OAuth2 token (Default value = None).
+        uri(str): DOI or URL of dataset to import.
+        name(str): Name to give imported dataset (Default value = "").
+        extract(bool): Whether to extract compressed dataset data (Default value = False).
+        yes(bool): Whether to skip user confirmation (Default value = False).
+        datadir(Optional[Path]): Dataset's data directory (Default value = None).
+        previous_dataset(Optional[Dataset]): Previously imported dataset version (Default value = None).
+        delete(bool): Whether to delete files that don't exist anymore (Default value = False).
+        gitlab_token(Optional[str]): Gitlab OAuth2 token (Default value = None).
     """
     from renku.core.dataset.dataset_add import add_to_dataset
 
+    client = get_client()
+
     def confirm_download(files):
         if yes:
             return
@@ -464,12 +511,12 @@ def import_dataset(
         deleted_paths = previous_paths - current_paths
 
         for path in deleted_paths:
-            delete_file(get_client().path / path, follow_symlinks=True)
+            delete_dataset_file(client.path / path, follow_symlinks=True)
 
     provider = ProviderFactory.get_import_provider(uri)
 
     try:
-        importer = provider.get_importer(uri, gitlab_token=gitlab_token, **kwargs)
+        importer = provider.get_importer(gitlab_token=gitlab_token, **kwargs)
         provider_dataset: ProviderDataset = importer.fetch_provider_dataset()
     except KeyError as e:
         raise errors.ParameterError(f"Could not process '{uri}'.\nUnable to fetch metadata: {e}")
@@ -487,6 +534,14 @@ def import_dataset(
     except (KeyError, LookupError):
         pass
 
+    if datadir and previous_dataset:
+        raise errors.ParameterError("Can't specify datadir when updating a previously imported dataset.")
+    elif datadir:
+        try:
+            datadir = get_safe_relative_path(datadir, client.path)
+        except ValueError as e:
+            raise errors.ParameterError("Datadir must be inside repository.") from e
+
     name = name or provider_dataset.name
 
     new_dataset = add_to_dataset(
@@ -499,6 +554,7 @@ def import_dataset(
         overwrite=True,
         total_size=calculate_total_size(importer.provider_dataset_files),
         clear_files_before=True,
+        datadir=datadir,
     )
 
     new_dataset.update_metadata_from(provider_dataset)
@@ -516,29 +572,35 @@ def import_dataset(
 @inject.autoparams()
 def update_datasets(
     names: List[str],
-    creators,
-    include,
-    exclude,
-    ref,
-    delete,
-    no_external,
-    update_all,
-    dry_run,
+    creators: Optional[str],
+    include: Optional[List[str]],
+    exclude: Optional[List[str]],
+    ref: Optional[str],
+    delete: bool,
+    no_external: bool,
+    no_local: bool,
+    no_remote: bool,
+    check_data_directory: bool,
+    update_all: bool,
+    dry_run: bool,
     client_dispatcher: IClientDispatcher,
     dataset_gateway: IDatasetGateway,
 ) -> Tuple[List[DatasetViewModel], List[DatasetFileViewModel]]:
     """Update dataset files.
 
     Args:
-        names: Names of datasets to update.
-        creators: Creators to filter dataset files by.
-        include: Include filter for paths to update.
-        exclude: Exclude filter for paths to update.
-        ref: Git reference to use for update.
-        delete: Whether to delete files that don't exist on remote anymore.
-        no_external: Whether to exclude external files from the update.
-        update_all: Whether to update all datasets.
-        dry_run: Whether to return a preview of what would be updated.
+        names(List[str]): Names of datasets to update.
+        creators(Optional[str]): Creators to filter dataset files by.
+        include(Optional[List[str]]): Include filter for paths to update.
+        exclude(Optional[List[str]]): Exclude filter for paths to update.
+        ref(Optional[str]): Git reference to use for update.
+        delete(bool): Whether to delete files that don't exist on remote anymore.
+        no_external(bool): Whether to exclude external files from the update.
+        no_local(bool): Whether to exclude local files from the update.
+        no_remote(bool): Whether to exclude remote files from the update.
+        check_data_directory(bool): Whether to check the dataset's data directory for new files.
+        update_all(bool): Whether to update all datasets.
+        dry_run(bool): Whether to return a preview of what would be updated.
         client_dispatcher(IClientDispatcher): Injected client dispatcher.
         dataset_gateway(IDatasetGateway): Injected dataset gateway.
     """
@@ -564,7 +626,7 @@ def update_datasets(
     names = names or [d.name for d in all_datasets]
 
     # NOTE: update imported datasets
-    if not include and not exclude:
+    if not include and not exclude and not no_remote:
         must_match_records = False
 
         for dataset in imported_datasets:
@@ -577,7 +639,7 @@ def update_datasets(
             except errors.DatasetProviderNotFound:
                 continue
 
-            record = provider.get_importer(uri)
+            record = provider.get_importer()
 
             if isinstance(provider, RenkuProvider) and dataset.version is not None:
                 tags = dataset_gateway.get_all_tags(dataset=dataset)
@@ -626,7 +688,12 @@ def update_datasets(
 
     # NOTE: Exclude all imported dataset from individual file filter
     records = filter_dataset_files(
-        names=names, creators=creators, include=include, exclude=exclude, ignore=[d.name for d in imported_datasets]
+        names=names,
+        creators=creators,
+        include=include,
+        exclude=exclude,
+        ignore=[d.name for d in imported_datasets],
+        check_data_directory=check_data_directory,
     )
 
     if not records:
@@ -661,15 +728,18 @@ def update_datasets(
         updated = update_external_files(client, external_files, dry_run=dry_run)
         updated_files.extend(updated)
 
-    if git_files:
+    if git_files and not no_remote:
         updated, deleted = update_dataset_git_files(files=git_files, ref=ref, delete=delete, dry_run=dry_run)
         updated_files.extend(updated)
         deleted_files.extend(deleted)
 
-    if local_files:
-        updated, deleted = update_dataset_local_files(records=local_files)
+    if local_files and not no_local:
+        updated, deleted, new = update_dataset_local_files(
+            records=local_files, check_data_directory=check_data_directory
+        )
         updated_files.extend(updated)
         deleted_files.extend(deleted)
+        updated_files.extend(new)
 
     if not dry_run:
         if deleted_files and not delete:
@@ -688,7 +758,8 @@ def update_datasets(
         communication.echo(message)
     else:
         for file in deleted_files:
-            file.date_removed = local_now()
+            if not file.date_removed:
+                file.date_removed = local_now()
 
     dataset_files_view_models = [
         DatasetFileViewModel.from_dataset_file(cast(DatasetFile, f), f.dataset) for f in updated_files + deleted_files
@@ -707,7 +778,7 @@ def show_dataset(name: str, tag: Optional[str] = None):
         dict: JSON dictionary of dataset details.
     """
     datasets_provenance = DatasetsProvenance()
-    dataset = datasets_provenance.get_by_name(name, strict=True)
+    dataset: Optional[Dataset] = datasets_provenance.get_by_name(name, strict=True)
 
     if tag is None:
         return DatasetDetailsJson().dump(dataset)
@@ -818,7 +889,7 @@ def move_files(
         for src, dst in files.items():
             src = src.relative_to(client.path)
             dst = dst.relative_to(client.path)
-            # NOTE: Files are moved at this point, so, we use can use dst
+            # NOTE: Files are moved at this point, so, we can use dst
             new_dataset_file = DatasetFile.from_path(client, dst)
 
             for dataset in datasets:
@@ -827,12 +898,17 @@ def move_files(
                     modified_datasets[dataset.name] = dataset
                     new_dataset_file.based_on = removed.based_on
                     new_dataset_file.source = removed.source
-                    if not to_dataset:
+
+                    if not to_dataset and (
+                        new_dataset_file.is_external
+                        or is_subpath(client.path / dst, client.path / dataset.get_datadir(client))
+                    ):
                         dataset.add_or_update_files(new_dataset_file)
 
                 # NOTE: Update dataset if it contains a destination that is being overwritten
                 modified = dataset.find_file(dst)
-                if modified:
+                added = is_subpath(client.path / dst, client.path / dataset.get_datadir(client))
+                if modified or added:
                     modified_datasets[dataset.name] = dataset
                     dataset.add_or_update_files(new_dataset_file)
 
@@ -853,14 +929,14 @@ def move_files(
 
 @inject.autoparams("client_dispatcher")
 def update_dataset_local_files(
-    client_dispatcher: IClientDispatcher, records: List[DynamicProxy]
-) -> Tuple[List[DynamicProxy], List[DynamicProxy]]:
+    client_dispatcher: IClientDispatcher, records: List[DynamicProxy], check_data_directory: bool
+) -> Tuple[List[DynamicProxy], List[DynamicProxy], List[DynamicProxy]]:
     """Update files metadata from the git history.
 
     Args:
         client_dispatcher(IClientDispatcher): Injected client dispatcher.
         records(List[DynamicProxy]): File records to update.
-
+        check_data_directory(bool): Whether to check the dataset's data directory for new files.
     Returns:
         Tuple[List[DynamicProxy], List[DynamicProxy]]: Tuple of updated and deleted file records.
     """
@@ -868,6 +944,7 @@ def update_dataset_local_files(
 
     updated_files: List[DynamicProxy] = []
     deleted_files: List[DynamicProxy] = []
+    new_files: List[DynamicProxy] = []
     progress_text = "Checking for local updates"
 
     try:
@@ -896,10 +973,18 @@ def update_dataset_local_files(
                 deleted_files.append(file)
             elif current_checksum != file.entity.checksum:
                 updated_files.append(file)
+            elif check_data_directory and not any(file.entity.path == f.entity.path for f in file.dataset.files):
+                datadir = file.dataset.get_datadir()
+                try:
+                    get_safe_relative_path(file.entity.path, datadir)
+                except ValueError:
+                    continue
+
+                new_files.append(file)
     finally:
         communication.finalize_progress(progress_text)
 
-    return updated_files, deleted_files
+    return updated_files, deleted_files, new_files
 
 
 def _update_datasets_files_metadata(
@@ -965,10 +1050,10 @@ def update_dataset_git_files(
             if url in visited_repos:
                 remote_repository, remote_client = visited_repos[url]
             else:
-                with communication.busy(msg="Cloning remote repository..."):
-                    remote_repository = clone_repository(
-                        url=url, path=get_cache_directory_for_repository(client=client, url=url), checkout_revision=ref
-                    )
+                communication.echo(msg="Cloning remote repository...")
+                remote_repository = clone_repository(
+                    url=url, path=get_cache_directory_for_repository(client=client, url=url), checkout_revision=ref
+                )
                 remote_client = LocalClient(path=remote_repository.path)
                 visited_repos[url] = remote_repository, remote_client
 
@@ -981,7 +1066,7 @@ def update_dataset_git_files(
 
             if not found:
                 if not dry_run and delete:
-                    delete_file(dst, follow_symlinks=True)
+                    delete_dataset_file(dst, follow_symlinks=True)
                     client.repository.add(dst, force=True)
                 deleted_files.append(file)
             elif changed:
@@ -989,7 +1074,7 @@ def update_dataset_git_files(
                     # Fetch file if it is tracked by Git LFS
                     remote_client.pull_paths_from_storage(remote_client.path / based_on.path)
                     if is_external_file(path=src, client_path=remote_client.path):
-                        delete_file(dst, follow_symlinks=True)
+                        delete_dataset_file(dst, follow_symlinks=True)
                         create_external_file(client=client, target=src.resolve(), path=dst)
                     else:
                         shutil.copy(src, dst)
@@ -1046,6 +1131,7 @@ def filter_dataset_files(
     exclude: Optional[List[str]] = None,
     ignore: Optional[List[str]] = None,
     immutable: bool = False,
+    check_data_directory: bool = False,
 ) -> List[DynamicProxy]:
     """Filter dataset files by specified filters.
 
@@ -1059,7 +1145,8 @@ def filter_dataset_files(
         exclude(Optional[List[str]]): Tuple containing patterns to which exclude from result (Default value = None).
         ignore(Optional[List[str]]): Ignored datasets (Default value = None).
         immutable(bool): Return immutable copies of dataset objects (Default value = False).
-
+        check_data_directory(bool): Whether to check for new files in dataset's data directory that aren't in the
+            dataset yet (Default value = False).
     Returns:
         List[DynamicProxy]: List of filtered files sorted by date added.
     """
@@ -1091,6 +1178,9 @@ def filter_dataset_files(
     records = []
     unused_names = set(names) if names is not None else set()
 
+    if ignore:
+        unused_names = unused_names - set(ignore)
+
     for dataset in dataset_gateway.get_all_active_datasets():
         if (names and dataset.name not in names) or (ignore and dataset.name in ignore):
             continue
@@ -1120,8 +1210,186 @@ def filter_dataset_files(
             record.client = client
             records.append(record)
 
+        if not check_data_directory:
+            continue
+
+        for root, _, files in os.walk(client.path / dataset.get_datadir()):
+            current_folder = Path(root)
+            for current_file in files:
+                file_path = get_safe_relative_path(current_folder / current_file, client.path)
+                if should_include(file_path) and not dataset.find_file(file_path):
+                    # New file in dataset folder
+                    record = DynamicProxy(DatasetFile.from_path(client, file_path))
+                    record.dataset = dataset
+                    record.client = client
+                    records.append(record)
+
     if unused_names:
         unused_names_str = ", ".join(unused_names)
         raise errors.ParameterError(f"These datasets don't exist: {unused_names_str}")
 
     return sorted(records, key=lambda r: r.date_added)
+
+
+@inject.autoparams("client_dispatcher", "storage_factory")
+def pull_external_data(
+    name: str, client_dispatcher: IClientDispatcher, storage_factory: IStorageFactory, location: Optional[Path] = None
+) -> None:
+    """Pull/copy data for an external storage to a dataset's data directory or a specified location.
+
+    Args:
+        name(str): Name of the dataset
+        location(Optional[Path]): A directory to copy data to (Default value = None).
+    """
+    client = client_dispatcher.current_client
+    datasets_provenance = DatasetsProvenance()
+
+    dataset = datasets_provenance.get_by_name(name=name, strict=True)
+
+    if not dataset.storage:
+        communication.warn(f"Dataset '{name}' doesn't have a storage backend")
+        return
+
+    # NOTE: Try to unmount the path in case it was mounted before
+    unmount_path(client.path / dataset.get_datadir())
+
+    create_symlinks = True
+    destination: Union[Path, str]
+
+    if location:
+        destination = get_absolute_path(location)
+    else:
+        stored_location = read_dataset_data_location(dataset=dataset)
+        if stored_location:
+            destination = stored_location
+        else:
+            destination = client.path
+            create_symlinks = False
+
+    provider = ProviderFactory.get_pull_provider(uri=dataset.storage)
+
+    credentials = S3Credentials(provider)
+    prompt_for_credentials(credentials)
+
+    storage = storage_factory.get_storage(provider=provider, credentials=credentials)
+    updated_files = []
+
+    for file in dataset.files:
+        path = Path(destination) / file.entity.path
+        path.parent.mkdir(parents=True, exist_ok=True)
+        # NOTE: Don't check if destination exists. ``IStorage.copy`` won't copy a file if it exists and is not modified.
+
+        if not file.source:
+            raise errors.DatasetImportError(f"Dataset file doesn't have a URI: {file.entity.path}")
+
+        with communication.busy(f"Copying {file.entity.path} ..."):
+            storage.copy(file.source, path)
+
+            # NOTE: Make files read-only since we don't support pushing data to the remote storage
+            os.chmod(path, 0o400)
+
+            if file.based_on and not file.based_on.checksum:
+                md5_hash = hash_file(path, hash_type="md5") or ""
+                file.based_on = RemoteEntity(checksum=md5_hash, url=file.based_on.url, path=file.based_on.path)
+
+            new_file = DynamicProxy(file)
+            new_file.dataset = dataset
+            updated_files.append(new_file)
+
+            if create_symlinks:
+                symlink_path = client.path / file.entity.path
+                symlink_path.parent.mkdir(parents=True, exist_ok=True)
+                create_symlink(path=path, symlink_path=symlink_path, overwrite=True)
+
+    # NOTE: Store location in metadata in case where we want to mount the external storage in the same location
+    store_dataset_data_location(dataset=dataset, location=location)
+
+    if updated_files:
+        _update_datasets_files_metadata(client, updated_files=updated_files, deleted_files=[], delete=False)
+
+
+def store_dataset_data_location(dataset: Dataset, location: Optional[Path]) -> None:
+    """Store data location for a dataset in the config file."""
+    client = get_client()
+
+    section = "dataset-locations"
+    key = dataset.name
+
+    if not location:
+        client.remove_value(section=section, key=key)
+    else:
+        client.set_value(section=section, key=key, value=get_absolute_path(location))
+
+
+def read_dataset_data_location(dataset: Dataset) -> Optional[str]:
+    """Read data location for a dataset in the config file."""
+    return get_client().get_value(section="dataset-locations", key=dataset.name, config_filter=ConfigFilter.LOCAL_ONLY)
+
+
+@inject.autoparams("client_dispatcher", "storage_factory")
+def mount_external_storage(
+    name: str,
+    existing: Optional[Path],
+    yes: bool,
+    storage_factory: IStorageFactory,
+) -> None:
+    """Mount an external storage to a dataset's data directory.
+
+    Args:
+        name(str): Name of the dataset
+        existing(Optional[Path]): An existing mount point to use instead of actually mounting the external storage.
+        yes(bool): Don't prompt when removing non-empty dataset's data directory.
+        storage_factory(IStorageFactory): Injected storage factory.
+    """
+    dataset, datadir = _get_dataset_with_external_storage(name=name)
+
+    # NOTE: Try to unmount the path in case it was mounted before
+    unmount_path(datadir)
+
+    if not is_path_empty(datadir) and not yes:
+        communication.confirm(
+            f"Dataset's data directory will be removed: {dataset.get_datadir()}. Do you want to continue?",
+            abort=True,
+            warning=True,
+        )
+
+    if existing:
+        create_symlink(path=existing, symlink_path=datadir, overwrite=True)
+        return
+
+    delete_path(datadir)
+    datadir.mkdir(parents=True, exist_ok=True)
+
+    provider = ProviderFactory.get_mount_provider(uri=dataset.storage)
+    credentials = S3Credentials(provider)
+    prompt_for_credentials(credentials)
+
+    storage = storage_factory.get_storage(provider=provider, credentials=credentials)
+    with communication.busy(f"Mounting {provider.uri}"):
+        storage.mount(datadir)
+
+
+def unmount_external_storage(name: str) -> None:
+    """Mount an external storage to a dataset's data directory.
+
+    Args:
+        name(str): Name of the dataset
+    """
+    _, datadir = _get_dataset_with_external_storage(name=name)
+    unmount_path(datadir)
+
+
+@inject.autoparams("client_dispatcher")
+def _get_dataset_with_external_storage(name: str, client_dispatcher: IClientDispatcher) -> Tuple[Dataset, Path]:
+
+    client = client_dispatcher.current_client
+    datasets_provenance = DatasetsProvenance()
+
+    dataset = datasets_provenance.get_by_name(name=name, strict=True)
+
+    if not dataset.storage:
+        raise errors.ParameterError(f"Dataset '{name}' doesn't have a storage backend")
+
+    datadir = client.path / dataset.get_datadir()
+
+    return dataset, datadir
