@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright 2018-2021 - Swiss Data Science Center (SDSC)
+# Copyright 2018-2022 - Swiss Data Science Center (SDSC)
 # A partnership between École Polytechnique Fédérale de Lausanne (EPFL) and
 # Eidgenössische Technische Hochschule Zürich (ETHZ).
 #
@@ -26,26 +26,26 @@ import time
 import uuid
 from contextlib import contextmanager
 from pathlib import Path
+from typing import List
 
 import attr
 
 from renku.core import errors
-from renku.core.utils.os import get_absolute_path
-from renku.core.utils.scm import shorten_message
-from renku.core.utils.urls import remove_credentials
+from renku.core.util.os import get_absolute_path
+from renku.core.util.urls import remove_credentials
 
 COMMIT_DIFF_STRATEGY = "DIFF"
 STARTED_AT = int(time.time() * 1e3)
 
 
-def prepare_commit(
-    client,
-    commit_only=None,
-    skip_dirty_checks=False,
-):
+def prepare_commit(client, commit_only=None, skip_dirty_checks=False, skip_staging: bool = False):
     """Gather information about repo needed for committing later on."""
     diff_before = set()
 
+    if skip_staging:
+        if not isinstance(commit_only, list) or len(commit_only) == 0:
+            raise errors.OperationError("Cannot use ``skip_staging`` without specifying files to commit.")
+
     if commit_only == COMMIT_DIFF_STRATEGY:
         if len(client.repository.staged_changes) > 0 or len(client.repository.unstaged_changes) > 0:
             client.repository.reset()
@@ -73,9 +73,11 @@ def finalize_commit(
     raise_if_empty=False,
     commit_message=None,
     abbreviate_message=True,
+    skip_staging: bool = False,
 ):
     """Commit modified/added paths."""
-    from renku.core.metadata.repository import Actor
+    from renku.core.util.git import shorten_message
+    from renku.infrastructure.repository import Actor
     from renku.version import __version__, version_url
 
     committer = Actor(name=f"renku {__version__}", email=version_url)
@@ -127,8 +129,10 @@ def finalize_commit(
     if abbreviate_message:
         commit_message = shorten_message(commit_message)
 
+    # NOTE: Only commit specified paths when skipping staging area
+    paths = commit_only if skip_staging else []
     # Ignore pre-commit hooks since we have already done everything.
-    client.repository.commit(commit_message + client.transaction_id, committer=committer, no_verify=True)
+    client.repository.commit(commit_message + client.transaction_id, committer=committer, no_verify=True, paths=paths)
 
 
 def prepare_worktree(
@@ -138,8 +142,8 @@ def prepare_worktree(
     commit=None,
 ):
     """Set up a Git worktree to provide isolation."""
-    from renku.core.metadata.repository import NULL_TREE
-    from renku.core.utils.contexts import Isolation
+    from renku.core.util.contexts import Isolation
+    from renku.infrastructure.repository import NULL_TREE
 
     path = path or tempfile.mkdtemp()
     branch_name = branch_name or "renku/run/isolation/" + uuid.uuid4().hex
@@ -147,16 +151,15 @@ def prepare_worktree(
     # TODO sys.argv
 
     if commit is NULL_TREE:
-        args = ["add", "--detach", path]
-        original_client.repository.run_git_command("worktree", *args)
+        original_client.repository.create_worktree(path, detach=True)
         client = attr.evolve(original_client, path=path)
         client.repository.run_git_command("checkout", "--orphan", branch_name)
         client.repository.remove("*", recursive=True, force=True)
     else:
-        args = ["add", "-b", branch_name, path]
+        revision = None
         if commit:
-            args.append(commit.hexsha)
-        original_client.repository.run_git_command("worktree", *args)
+            revision = commit.hexsha
+        original_client.repository.create_worktree(path, branch=branch_name, reference=revision)
         client = attr.evolve(original_client, path=path)
 
     client.repository.get_configuration = original_client.repository.get_configuration
@@ -204,7 +207,7 @@ def finalize_worktree(
         raise errors.FailedMerge(client.repository, branch_name, merge_args)
 
     if delete:
-        client.repository.run_git_command("worktree", "remove", path)
+        client.repository.remove_worktree(path)
 
         if new_branch:
             # delete the created temporary branch
@@ -254,9 +257,8 @@ def _clean_streams(repository, mapped_streams):
         if path not in repository.files:
             os.remove(absolute_path)
         else:
-            with open(path, "wb") as output_file:
-                checksum = repository.get_object_hash(path=absolute_path, revision="HEAD")
-                repository.copy_content_to_file(path=absolute_path, checksum=checksum, output_file=output_file)
+            checksum = repository.get_object_hash(path=absolute_path, revision="HEAD")
+            repository.copy_content_to_file(path=absolute_path, checksum=checksum, output_path=path)
 
 
 def _expand_directories(paths):
@@ -284,7 +286,7 @@ class GitCore:
 
     def __attrs_post_init__(self):
         """Initialize computed attributes."""
-        from renku.core.metadata.repository import Repository
+        from renku.infrastructure.repository import Repository
 
         #: Create an instance of a Git repository for the given path.
         try:
@@ -292,6 +294,10 @@ class GitCore:
         except errors.GitError:
             self.repository = None
 
+    def __del__(self):
+        if self.repository:
+            self.repository.close()
+
     @property
     def modified_paths(self):
         """Return paths of modified files."""
@@ -314,7 +320,7 @@ class GitCore:
             for path in itertools.chain(self.repository.files, self.repository.untracked_files)
         ]
 
-    def find_ignored_paths(self, *paths):
+    def find_ignored_paths(self, *paths) -> List[str]:
         """Return ignored paths matching ``.gitignore`` file."""
         return self.repository.get_ignored_paths(*paths)
 
@@ -405,7 +411,7 @@ class GitCore:
     @contextmanager
     def worktree(self, path=None, branch_name=None, commit=None, merge_args=("--ff-only",)):
         """Create new worktree."""
-        from renku.core.metadata.repository import NULL_TREE
+        from renku.infrastructure.repository import NULL_TREE
 
         delete = branch_name is None
         new_branch = commit is not NULL_TREE
