@@ -30,6 +30,7 @@ import marshmallow
 
 from renku.core import errors
 from renku.core.util.datetime8601 import fix_datetime, local_now, parse_date
+from renku.core.util.dispatcher import get_client
 from renku.core.util.git import get_entity_from_revision
 from renku.core.util.metadata import is_external_file
 from renku.core.util.urls import get_path, get_slug
@@ -38,6 +39,7 @@ from renku.infrastructure.immutable import Immutable, Slots
 from renku.infrastructure.persistent import Persistent
 
 if TYPE_CHECKING:
+    from renku.core.management.client import LocalClient
     from renku.domain_model.entity import Entity
     from renku.domain_model.provenance.agent import Person
     from renku.domain_model.provenance.annotation import Annotation
@@ -262,7 +264,15 @@ class DatasetFile(Slots):
         cls, client, path: Union[str, Path], source=None, based_on: Optional[RemoteEntity] = None
     ) -> "DatasetFile":
         """Return an instance from a path."""
-        entity = get_entity_from_revision(repository=client.repository, path=path, bypass_cache=True)
+        from renku.domain_model.entity import NON_EXISTING_ENTITY_CHECKSUM, Entity
+
+        # NOTE: Data is added from an external storage and isn't pulled yet
+        if based_on and not (client.path / path).exists():
+            checksum = based_on.checksum if based_on.checksum else NON_EXISTING_ENTITY_CHECKSUM
+            id = Entity.generate_id(checksum=checksum, path=path)
+            entity = Entity(id=id, checksum=checksum, path=path)
+        else:
+            entity = get_entity_from_revision(repository=client.repository, path=path, bypass_cache=True)
 
         is_external = is_external_file(path=path, client_path=client.path)
         return cls(entity=entity, is_external=is_external, source=source, based_on=based_on)
@@ -316,12 +326,15 @@ class Dataset(Persistent):
     """Represent a dataset."""
 
     date_modified: Optional[datetime] = None  # type: ignore
+    storage: Optional[str] = None
+    datadir: Optional[str] = None
 
     def __init__(
         self,
         *,
         annotations: Optional[List["Annotation"]] = None,
         creators: Optional[List["Person"]] = None,
+        datadir: Optional[Path] = None,
         dataset_files: Optional[List[DatasetFile]] = None,
         date_created: Optional[datetime] = None,
         date_published: Optional[datetime] = None,
@@ -338,6 +351,7 @@ class Dataset(Persistent):
         name: Optional[str] = None,
         project_id: Optional[str] = None,
         same_as: Optional[Url] = None,
+        storage: Optional[str] = None,
         title: Optional[str] = None,
         version: Optional[str] = None,
     ):
@@ -377,10 +391,14 @@ class Dataset(Persistent):
         self.license: Optional[str] = license
         self.project_id: Optional[str] = project_id
         self.same_as: Optional[Url] = same_as
+        self.storage: Optional[str] = storage
         self.title: Optional[str] = title
         self.version: Optional[str] = version
         self.annotations: List["Annotation"] = annotations or []
 
+        if datadir:
+            self.datadir: Optional[str] = str(datadir)
+
     @staticmethod
     def generate_id(identifier: str) -> str:
         """Generate an identifier for Dataset."""
@@ -420,6 +438,14 @@ class Dataset(Persistent):
         """Comma-separated list of keywords associated with dataset."""
         return ", ".join(self.keywords)
 
+    def get_datadir(self, client: Optional["LocalClient"] = None) -> Path:
+        """Return dataset's data directory."""
+        if self.datadir:
+            return Path(self.datadir)
+
+        client = client or get_client()
+        return Path(os.path.join(client.data_dir, self.name))
+
     def __repr__(self) -> str:
         return f"<Dataset {self.identifier} {self.name}>"
 
@@ -628,9 +654,21 @@ class DatasetDetailsJson(marshmallow.Schema):
     description = marshmallow.fields.String()
     keywords = marshmallow.fields.List(marshmallow.fields.String())
     identifier = marshmallow.fields.String()
+    storage = marshmallow.fields.String()
 
     annotations = marshmallow.fields.List(marshmallow.fields.Nested(AnnotationJson))
 
+    data_directory = marshmallow.fields.Method("get_datadir")
+
+    def get_datadir(self, obj):
+        """Get data directory."""
+        if isinstance(obj, dict):
+            return str(obj.get("datadir_path", obj.get("datadir", "")))
+        if hasattr(obj, "datadir_path"):
+            return obj.datadir_path
+
+        return str(obj.get_datadir())
+
 
 class DatasetFileDetailsJson(marshmallow.Schema):
     """Serialize dataset files to a response object."""
@@ -664,14 +702,9 @@ class ImageObjectRequestJson(marshmallow.Schema):
     mirror_locally = marshmallow.fields.Bool(default=False)
 
 
-def get_dataset_data_dir(client, dataset_name: str) -> str:
-    """Return default data directory for a dataset."""
-    return os.path.join(client.data_dir, dataset_name)
-
-
 def get_file_path_in_dataset(client, dataset: Dataset, dataset_file: DatasetFile) -> Path:
     """Return path of a file relative to dataset's data dir."""
     try:
-        return (client.path / dataset_file.entity.path).relative_to(get_dataset_data_dir(client, dataset.name))
+        return (client.path / dataset_file.entity.path).relative_to(client.path / dataset.get_datadir(client))
     except ValueError:  # NOTE: File is not in the dataset's data dir
         return Path(dataset_file.entity.path)
