@@ -17,75 +17,38 @@
 # limitations under the License.
 r"""Renku CLI commands for handling of datasets.
 
-Manipulating datasets
-~~~~~~~~~~~~~~~~~~~~~
+Description
+~~~~~~~~~~~
 
-.. image:: ../_static/asciicasts/dataset-create.delay.gif
+Create, edit and manage the datasets in your Renku project.
+
+This is a core feature of Renku. You might want to go through the examples
+listed below to get an idea of how you can create, import, and edit datasets.
+
+Commands and options
+~~~~~~~~~~~~~~~~~~~~
+
+.. rst-class:: cli-reference-commands
+
+.. click:: renku.ui.cli.dataset:dataset
+   :prog: renku dataset
+   :nested: full
+
+Examples
+~~~~~~~~
+
+Create an empty dataset inside a Renku project:
+
+.. image:: ../../_static/asciicasts/dataset-create.delay.gif
    :width: 850
    :alt: Create a Dataset
 
-Creating an empty dataset inside a Renku project:
-
-.. code-block:: console
-
-    $ renku dataset create my-dataset
-    Creating a dataset ... OK
-
 .. cheatsheet::
    :group: Datasets
    :command: $ renku dataset create <dataset>
    :description: Create a new dataset.
    :extended:
 
-You can pass the following options to this command to set various metadata for
-the dataset.
-
-+-------------------+------------------------------------------------------+
-| Option            | Description                                          |
-+===================+======================================================+
-| -t, --title       | A human-readable title for the dataset.              |
-+-------------------+------------------------------------------------------+
-| -d, --description | Dataset's description.                               |
-+-------------------+------------------------------------------------------+
-| -c, --creator     | Creator's name, email, and an optional affiliation.  |
-|                   | Accepted format is                                   |
-|                   | 'Forename Surname <email> [affiliation]'. Pass       |
-|                   | multiple times for a list of creators.               |
-+-------------------+------------------------------------------------------+
-| -k, --keyword     | Dataset's keywords. Pass multiple times for a list   |
-|                   | of keywords.                                         |
-+-------------------+------------------------------------------------------+
-| -m, --metadata    | Path to file containing custom JSON-LD metadata to   |
-|                   | be added to the dataset.                             |
-+-------------------+------------------------------------------------------+
-
-Editing a dataset's metadata:
-
-.. image:: ../_static/asciicasts/dataset-edit.delay.gif
-   :width: 850
-   :alt: Editing a Dataset
-
-Use the ``edit`` sub-command to change metadata of a dataset. You can edit the same
-set of metadata as the create command by passing the options described in the
-table above. You can also use the ``-u/--unset`` options with one of the values
-from ``creators`` (short ``c``), ``keywords`` (short ``k``) or ``metadata``
-(short ``m``) to delete the respective values from the dataset.
-
-.. code-block:: console
-
-    $ renku dataset edit my-dataset --title 'New title' --unset keywords
-    Successfully updated: title.
-
-Listing all datasets:
-
-.. code-block:: console
-
-    $ renku dataset ls
-    ID        NAME           TITLE          VERSION
-    --------  -------------  -------------  ---------
-    0ad1cb9a  some-dataset   Some Dataset
-    9436e36c  my-dataset     My Dataset
-
 .. cheatsheet::
    :group: Datasets
    :command: $ renku dataset ls
@@ -141,19 +104,29 @@ Deleting a dataset:
    :description: Remove a dataset.
    :extended:
 
+Creating a dataset with a storage backend:
+
+By passing a storage URI with the ``--storage`` option, you can tell Renku that
+the data for the dataset is stored in a remote storage. At the moment, Renku
+supports only S3 backends. For example:
+
+.. code-block:: console
+
+    $ renku dataset create s3-data --storage s3://bucket-name/path
+
+Renku prompts for your S3 credentials and can store them for future uses.
+
+.. note:: Data directory for datasets that have a storage backend is ignored by
+    Git. This is needed to avoid committing pulled data from a remote storage to Git.
 
 Working with data
 ~~~~~~~~~~~~~~~~~
 
-.. image:: ../_static/asciicasts/dataset-add.delay.gif
-   :width: 850
-   :alt: Add data to a Dataset
-
 Adding data to the dataset:
 
-.. code-block:: console
-
-    $ renku dataset add my-dataset http://data-url
+.. image:: ../../_static/asciicasts/dataset-add.delay.gif
+   :width: 850
+   :alt: Add data to a Dataset
 
 .. cheatsheet::
    :group: Datasets
@@ -273,7 +246,8 @@ will prevent them from being updated.
    :extended:
 
 The update command also checks for file changes in the project and updates
-datasets' metadata accordingly.
+datasets' metadata accordingly. You can automatically add new files from
+the dataset's data directory by using the ``--check-data-directory`` flag.
 
 You can limit the scope of updated files by specifying dataset names, using
 ``--include`` and ``--exclude`` to filter based on file names, or using
@@ -381,7 +355,7 @@ to the import command:
 
 Importing data from an external provider:
 
-.. image:: ../_static/asciicasts/dataset-import.delay.gif
+.. image:: ../../_static/asciicasts/dataset-import.delay.gif
    :width: 850
    :alt: Import a Dataset
 
@@ -396,6 +370,9 @@ or ``doi:10.5281/zenodo.3352150``) and full URLs (e.g.
 ``http://zenodo.org/record/3352150``). A tag with the remote version of the
 dataset is automatically created.
 
+You can change the directory a dataset is imported to by using the
+``--datadir`` option.
+
 .. cheatsheet::
    :group: Datasets
    :command: $ renku dataset import <uri>
@@ -544,6 +521,7 @@ Unlink all files from a dataset:
 """
 
 import json
+import os
 from pathlib import Path
 
 import click
@@ -579,7 +557,7 @@ def dataset():
     "-c",
     "--columns",
     type=click.STRING,
-    default="id,name,title,version",
+    default="id,name,title,version,datadir",
     metavar="<columns>",
     help="Comma-separated list of column to display: {}.".format(", ".join(DATASETS_COLUMNS.keys())),
     show_default=True,
@@ -616,7 +594,14 @@ def list_dataset(format, columns):
     help="Custom metadata to be associated with the dataset.",
 )
 @click.option("-k", "--keyword", default=None, multiple=True, type=click.STRING, help="List of keywords.")
-def create(name, title, description, creators, metadata, keyword):
+@click.option("-s", "--storage", default=None, type=click.STRING, help="URI of the storage backend.")
+@click.option(
+    "--datadir",
+    default=None,
+    type=click.Path(),
+    help="Dataset's data directory (defaults to 'data/<dataset name>').",
+)
+def create(name, title, description, creators, metadata, keyword, storage, datadir):
     """Create an empty dataset in the current repo."""
     from renku.command.dataset import create_dataset_command
     from renku.core.util.metadata import construct_creators
@@ -644,6 +629,8 @@ def create(name, title, description, creators, metadata, keyword):
             creators=creators,
             keywords=keyword,
             custom_metadata=custom_metadata,
+            storage=storage,
+            datadir=datadir,
         )
     )
 
@@ -771,6 +758,7 @@ def show(tag, name):
 
     click.echo(click.style("Name: ", bold=True, fg=color.MAGENTA) + click.style(ds["name"], bold=True))
     click.echo(click.style("Created: ", bold=True, fg=color.MAGENTA) + (ds.get("created_at", "") or ""))
+    click.echo(click.style("Data Directory: ", bold=True, fg=color.MAGENTA) + str(ds.get("data_directory", "") or ""))
 
     creators = []
     for creator in ds.get("creators", []):
@@ -784,6 +772,8 @@ def show(tag, name):
 
     click.echo(click.style("Version: ", bold=True, fg=color.MAGENTA) + (ds.get("version") or ""))
 
+    click.echo(click.style("Storage: ", bold=True, fg=color.MAGENTA) + (ds.get("storage") or ""))
+
     click.echo(click.style("Annotations: ", bold=True, fg=color.MAGENTA))
     if ds["annotations"]:
         click.echo(json.dumps(ds.get("annotations", ""), indent=2))
@@ -810,23 +800,39 @@ def add_provider_options(*param_decls, **attrs):
 @click.option("-o", "--overwrite", is_flag=True, help="Overwrite existing files.")
 @click.option("-c", "--create", is_flag=True, help="Create dataset if it does not exist.")
 @click.option("-d", "--destination", default="", help="Destination directory within the dataset path")
+@click.option(
+    "--datadir",
+    default=None,
+    type=click.Path(),
+    help="Dataset's data directory (defaults to 'data/<dataset name>').",
+)
 @add_provider_options()
-def add(name, urls, external, force, overwrite, create, destination, **kwargs):
+def add(name, urls, force, overwrite, create, destination, datadir, **kwargs):
     """Add data to a dataset."""
     from renku.command.dataset import add_to_dataset_command
     from renku.ui.cli.utils.callback import ClickCallback
 
     communicator = ClickCallback()
-    add_to_dataset_command().with_communicator(communicator).build().execute(
-        urls=urls,
-        dataset_name=name,
-        external=external,
-        force=force,
-        overwrite=overwrite,
-        create=create,
-        destination=destination,
-        **kwargs,
+    result = (
+        add_to_dataset_command()
+        .with_communicator(communicator)
+        .build()
+        .execute(
+            urls=urls,
+            dataset_name=name,
+            force=force,
+            overwrite=overwrite,
+            create=create,
+            destination=destination,
+            datadir=datadir,
+            **kwargs,
+        )
     )
+
+    dataset = result.output
+    if dataset.storage:
+        communicator.info(f"To download files from the remote storage use 'renku dataset pull {dataset.name}'")
+
     click.secho("OK", fg=color.GREEN)
 
 
@@ -1006,8 +1012,14 @@ def import_provider_options(*param_decls, **attrs):
 @click.option("--short-name", "--name", "name", default=None, help="A convenient name for dataset.")
 @click.option("-x", "--extract", is_flag=True, help="Extract files before importing to dataset.")
 @click.option("-y", "--yes", is_flag=True, help="Bypass download confirmation.")
+@click.option(
+    "--datadir",
+    default=None,
+    type=click.Path(),
+    help="Dataset's data directory (defaults to 'data/<dataset name>').",
+)
 @import_provider_options()
-def import_(uri, name, extract, yes, **kwargs):
+def import_(uri, name, extract, yes, datadir, **kwargs):
     """Import data from a 3rd party provider or another renku project.
 
     Supported providers: [Dataverse, Renku, Zenodo]
@@ -1017,14 +1029,15 @@ def import_(uri, name, extract, yes, **kwargs):
 
     communicator = ClickCallback()
     import_dataset_command().with_communicator(communicator).build().execute(
-        uri=uri, name=name, extract=extract, yes=yes, **kwargs
+        uri=uri, name=name, extract=extract, yes=yes, datadir=datadir, **kwargs
     )
 
     click.secho(" " * 79 + "\r", nl=False)
     click.secho("OK", fg=color.GREEN)
 
 
-@dataset.command("update")
+@dataset.command()
+@click.pass_context
 @click.argument("names", nargs=-1, shell_complete=_complete_datasets)
 @click.option(
     "--creators",
@@ -1036,9 +1049,35 @@ def import_(uri, name, extract, yes, **kwargs):
 @click.option("--delete", is_flag=True, help="Delete local files that are deleted from remote.")
 @click.option("-e", "--external", is_flag=True, help="Deprecated")
 @click.option("--no-external", is_flag=True, help="Skip updating external data.")
+@click.option("--no-local", is_flag=True, help="Skip updating local files.")
+@click.option("--no-remote", is_flag=True, help="Skip updating remote files.")
+@click.option("-c", "--check-data-directory", is_flag=True, help="Check datasets' data directories for new files.")
 @click.option("--all", "-a", "update_all", is_flag=True, default=False, help="Update all datasets.")
 @click.option("-n", "--dry-run", is_flag=True, help="Show what would have been changed")
-def update(names, creators, include, exclude, ref, delete, external, no_external, update_all, dry_run):
+@click.option(
+    "--plain",
+    is_flag=True,
+    default=False,
+    help="Show result as one entry per line for machine readability."
+    " 'd' = dataset update, 'f' = file update, 'r' = file removed.",
+)
+def update(
+    ctx,
+    names,
+    creators,
+    include,
+    exclude,
+    ref,
+    delete,
+    external,
+    no_external,
+    no_local,
+    no_remote,
+    check_data_directory,
+    update_all,
+    dry_run,
+    plain,
+):
     """Updates files in dataset from a remote Git repo."""
     from renku.command.dataset import update_datasets_command
     from renku.core import errors
@@ -1060,7 +1099,7 @@ def update(names, creators, include, exclude, ref, delete, external, no_external
         raise errors.ParameterError("Cannot pass --include/--exclude with -a/--all")
 
     result = (
-        update_datasets_command()
+        update_datasets_command(dry_run=dry_run)
         .with_communicator(communicator)
         .build()
         .execute(
@@ -1071,6 +1110,9 @@ def update(names, creators, include, exclude, ref, delete, external, no_external
             ref=ref,
             delete=delete,
             no_external=no_external,
+            no_local=no_local,
+            no_remote=no_remote,
+            check_data_directory=check_data_directory,
             update_all=update_all,
             dry_run=dry_run,
         )
@@ -1086,23 +1128,81 @@ def update(names, creators, include, exclude, ref, delete, external, no_external
             return tabulate(collection=records, columns="path,dataset,external", columns_mapping=columns)
 
         if not datasets and not dataset_files:
-            click.secho("Everything is up-to-date", fg=color.GREEN)
+            if not plain:
+                click.secho("Everything is up-to-date", fg=color.GREEN)
             return
 
         if datasets:
-            names = "\n\t".join(sorted([d.name for d in datasets]))
-            click.echo(f"The following imported datasets will be updated:\n\t{names}\n")
+            ds_names = sorted([d.name for d in datasets])
+            if plain:
+                ds_names = [f"d {n}" for n in ds_names]
+                click.echo(os.linesep.join(ds_names) + os.linesep)
+            else:
+                names = "\n\t".join(ds_names)
+                click.echo(f"The following imported datasets will be updated:\n\t{names}\n")
 
         if not dataset_files:
             return
 
         files = [f for f in dataset_files if not f.deleted]
         if files:
-            files = get_dataset_files(files)
-            click.echo(f"The following files will be updated:\n\n{files}\n")
+            if plain:
+                files = [f"f {f.path} {f.dataset.name}" for f in files]
+                click.echo(os.linesep.join(files) + os.linesep)
+            else:
+                files = get_dataset_files(files)
+                click.echo(f"The following files will be updated:\n\n{files}\n")
 
         deleted_files = [f for f in dataset_files if f.deleted]
         if deleted_files:
-            files = get_dataset_files(deleted_files)
-            message = " (pass '--delete' to remove them from datasets' metadata)" if not delete else ""
-            click.echo(f"The following files will be deleted{message}:\n\n{files}\n")
+            if plain:
+                files = [f"r {f.path} {f.dataset.name}" for f in deleted_files]
+                click.echo(os.linesep.join(files))
+            else:
+                files = get_dataset_files(deleted_files)
+                message = " (pass '--delete' to remove them from datasets' metadata)" if not delete else ""
+                click.echo(f"The following files will be deleted{message}:\n\n{files}\n")
+
+        ctx.exit(1)
+
+
+@dataset.command(hidden=True)
+@click.argument("name", shell_complete=_complete_datasets)
+@click.option(
+    "-l",
+    "--location",
+    default=None,
+    type=click.Path(exists=False, file_okay=False, writable=True),
+    help="A directory to copy data to, instead of the dataset's data directory.",
+)
+def pull(name, location):
+    """Pull data from an external storage."""
+    from renku.command.dataset import pull_external_data_command
+    from renku.ui.cli.utils.callback import ClickCallback
+
+    communicator = ClickCallback()
+    pull_external_data_command().with_communicator(communicator).build().execute(name=name, location=location)
+
+
+@dataset.command(hidden=True)
+@click.argument("name", shell_complete=_complete_datasets)
+@click.option(
+    "-e",
+    "--existing",
+    default=None,
+    type=click.Path(exists=True, file_okay=False),
+    help="Use an existing mount point instead of mounting the remote storage.",
+)
+@click.option("-u", "--unmount", is_flag=True, help="Unmount dataset's external storage.")
+@click.option("-y", "--yes", is_flag=True, help="No prompt when removing non-empty dataset's data directory.")
+def mount(name, existing, unmount, yes):
+    """Mount an external storage in the dataset's data directory."""
+    from renku.command.dataset import mount_external_storage_command
+    from renku.ui.cli.utils.callback import ClickCallback
+
+    command = mount_external_storage_command(unmount=unmount).with_communicator(ClickCallback()).build()
+
+    if unmount:
+        command.execute(name=name)
+    else:
+        command.execute(name=name, existing=existing, yes=yes)
